﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using Telerik.Pivot.Core;
using Telerik.Pivot.Core.Aggregates;
using Telerik.Pivot.Core.Filtering;
using Telerik.WinControls;
using Telerik.WinControls.RadControlSpy;
using Telerik.WinControls.UI;

namespace RadPivotGridUI
{
    public partial class Form1 : Form
    {
        List<Order> orders = new List<Order>();
        LocalDataSourceProvider provider;

        public Form1()
        {
            InitializeComponent();

            new RadControlSpyForm().Show();

            this.radPivotGrid1.GroupDescriptorElementCreating += radPivotGrid1_GroupDescriptorElementCreating;
        }

        LabelGroupFilter filter;
        SetConditionHashCollection filteredItems;
        private void radPivotGrid1_GroupDescriptorElementCreating(object sender, GroupDescriptorElementCreatingEventArgs e)
        {

            if ((e.GroupDescriptorElement.GroupDescriptor.GroupFilter as LabelGroupFilter) != null && e.GroupDescriptorElement.Text == "Week")
            {
                this.filter = (LabelGroupFilter)e.GroupDescriptorElement.GroupDescriptor.GroupFilter;
                PropertyInfo pi = ((ILabelGroupFilter)e.GroupDescriptorElement.GroupDescriptor.GroupFilter).Condition.GetType().GetProperty("Items", BindingFlags.Instance | BindingFlags.Public);
                this.filteredItems = pi.GetValue(((ILabelGroupFilter)e.GroupDescriptorElement.GroupDescriptor.GroupFilter).Condition) as SetConditionHashCollection;
            }

            e.GroupDescriptorElement.FilterPopup.PopupOpened -= FilterPopup_PopupOpened;
            e.GroupDescriptorElement.FilterPopup.PopupOpened += FilterPopup_PopupOpened;
        }

        private void FilterPopup_PopupOpening(object sender, CancelEventArgs args)
        {
            PivotGroupFilterPopup filterPopup = (PivotGroupFilterPopup)sender;
            RadTreeView tree = filterPopup.TreeViewMenuItem.TreeElement.TreeView;
        }

        private void FilterPopup_PopupOpened(object sender, EventArgs args)
        {
            PivotGroupFilterPopup filterPopup = (PivotGroupFilterPopup)sender;
            RadTreeView tree = filterPopup.TreeViewMenuItem.TreeElement.TreeView;
            if (this.filter != null && filterPopup.GroupDescriptorElement.Text == "Day")
            {
                List<string> names = new List<string>();
                foreach (object item in this.filteredItems)
                {
                    names.Add(item.ToString());
                }

                var rows = this.radPivotGrid1.PivotGridElement.GetRowGroups();
                List<int> indices = new List<int>();
                foreach (var node in rows)
                {
                    if (node.Group != null && node.Group.Parent != null && names.Contains(node.Group.Parent.Name.ToString()))
                    {
                        foreach (RadTreeNode n in tree.Nodes[0].Nodes)
                        {
                            if (n.Name == node.Name)
                            {
                                indices.Add(n.Index);
                            }

                            n.Visible = false; 
                        }

                        for (int i = 0; i < indices.Count; i++)
                        {
                            tree.Nodes[0].Nodes[indices[i]].Visible = true;
                        }
                    }
                }
            }
        }

        private void radPivotGrid1_FilterDescriptorElementCreating(object sender, FilterDescriptorElementCreatingEventArgs e)
        {
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            this.LoadData();

            this.provider = new LocalDataSourceProvider() { ItemsSource = orders };
            provider.ColumnGroupDescriptions.Add(new PropertyGroupDescription()
            {
                PropertyName = "Product"
            });
            provider.RowGroupDescriptions.Add(new PropertyGroupDescription()
            {
                PropertyName = "Promotion"
            });
            provider.RowGroupDescriptions.Add(new DateTimeGroupDescription()
            {
                PropertyName = "Date",
                Step = DateTimeStep.Week,
                SortOrder = Telerik.Pivot.Core.SortOrder.Ascending
            });
            provider.RowGroupDescriptions.Add(new DateTimeGroupDescription()
            {
                PropertyName = "Date",
                Step = DateTimeStep.Day,
                SortOrder = Telerik.Pivot.Core.SortOrder.Ascending
            });

            provider.AggregateDescriptions.Add(new PropertyAggregateDescription() { PropertyName = "Quantity", AggregateFunction = AggregateFunctions.Sum });
            provider.AggregateDescriptions.Add(new PropertyAggregateDescription() { PropertyName = "Net", AggregateFunction = AggregateFunctions.Sum });
            provider.AggregatesPosition = PivotAxis.Rows;
            provider.AggregatesLevel = 21;

            this.radPivotGrid1.PivotGridElement.DataProvider = provider;
            this.radPivotGrid1.PivotGridElement.BestFitHelper.BestFitColumns();
            this.radPivotGrid1.PivotGridElement.BestFitHelper.BestFitRowHeaders();
        }

        private void LoadData()
        {
            Stream stream = System.Reflection.Assembly.GetAssembly(this.GetType()).GetManifestResourceStream("RadPivotGridUI.PivotData.txt");

            using (StreamReader streamReader = new StreamReader(stream))
            {
                while (streamReader.Peek() != -1)
                {
                    string[] items = streamReader.ReadLine().Split('\t');
                    Order o = new Order()
                    {
                        Date = DateTime.Parse(items[0], System.Globalization.CultureInfo.InvariantCulture),
                        Product = items[1],
                        Quantity = int.Parse(items[2], System.Globalization.CultureInfo.InvariantCulture),
                        Net = double.Parse(items[3], System.Globalization.CultureInfo.InvariantCulture),
                        Promotion = items[4],
                        Advertisement = items[5]
                    };

                    orders.Add(o);
                }
            }
        }
    }
}
